package psimjava;

/**
 * This class controls the declaration, allocation and deallocation
 * resources.
 * <P>Psim-J Object-oriented simulation, process interaction approach.
 * <P> (C)Jose M. Garrido. June 2000, 2003, 2004

 * <P>If a process requesting a certain number of resource items in the
 * resource pool (of type 'Bin') are not available, then the process is
 * enqueued into the corresponding priority queue. When a process releases
 * a number of resource items, then the queues are examined for waiting
 * process.

 * <P> File: Bin.java
 */
public class Bin {
    private String title;
    private Pqueue q; // priority queue for waiting processes
    private Rqueue qq;
    private int available; // nr of available resource objects
    private long prodr; // accum bin resources and wait time
    private long prodn;
    private long freep;
    private double wait_time; // accum wait time
    private long free_int;
    private long tot_req;
    private static double lastev = 0.0;
    private double rlastev;
    private static int num_req = 0;
    private static int numproc = 0;
    private int obs_bin;
    private int max_bin;
    private int max_proc;
    //
    /**
     * Constructor initializes Bin object with a title and the number of available
     * resource items.
     * @param name title of Bin object created
     * @param n initial number of Bin items
     */
    public Bin(String name, int n) {
        title = new String(name);
        available = n; // initial size of bin
        q = new Pqueue(name, Pobject.MAX_PRIORITIES); // initialize priority queue
        qq = new Rqueue(Pobject.MAX_PRIORITIES); // initialize priority queue
        prodr = 0;
        prodn = 0;
        freep = 0;
        wait_time = 0;
        free_int = 0;
        tot_req = 0;
        rlastev = 0; // last event
        obs_bin = 0; // number of completed calls to the take operation
        max_bin = 0; // maximum number of resources in bin
        Simulation.pt_bin.addnode(this);
    }

    //
    /**
     * This method removes 'num' resource items from the Bin object pool,
     * if there are sufficient items in the pool it returns normally.
     * If there are not sufficient resource items available, the calling
     * process is suspended (to wait for available items).
     * @param num number of items to wirhdraw from Bin pool.
     */
    public void take(int num) {
        boolean take_cond;
        take_cond = false;
        while (!take_cond) {
            take_cond = btake(num);
        }
    }

    //
    private boolean btake(int num) { // process assumed 'running'
        int reqres;
        int card;
        Process lpr;
        int resheld;
        int pprio; // priority of process
        Process pp = (Process) StaticSync.this_process();
        if (num <= 0) {
            Pobject.process_error(Pobject.E_BINNRR);
        }
        else {
            // still pend req bin
            // System.out.println("Get res req for "+pp.get_name());
            reqres = qq.get_res_req(pp);
            // System.out.println("After get_res_req");
            if (num > available) {
                if (reqres > 0) {
                    // not sufficient resources for this process
                    // put back process with its resource request num
                    // at head of queue with priority
                    q.pback(pp);
                }
                else {
                    q.into(pp);
                    qq.enqueue(pp, 0, num);
                }
                wait_bin(num);
                // System.out.println("Bin:take " + title + " suspend process " +
                //       pp.get_name());
                pp.deactivate(pp); // passivate process
                return false;
            }
            else { // ok, get resources
                // if this process has been waiting to take resources
                // or if the waiting-processes queue is empty
                card = q.length();
                if (reqres > 0 || card == 0) {
                    available = available - num; // update number of resources
                    // get next process in queue waiting for resources
                    // delete pointer of current process for res requested
                    pprio = pp.get_prio();
                    // System.out.println(title+" remove object");
                    qq.rem_obj(pprio);
                    // System.out.println(title+ " after removing object");
                    wait_bin( -num); // for reporting req resources
                    use_bin(); // for reporting avg available resources
                    // System.out.println( title + " activating next process");
                    if (q.length() > 0) {
                        lpr = (Process) q.out(); // process with highest pr
                        // System.out.println(title + " activating "+
                        //       lpr.get_name());
                        StaticSync.reactivate(lpr); // promote to event list
                    }
                    obs_bin++; // completed call to take
                    // System.out.println(title+"Completed call: take");
                    return true;
                }
                else { // if process has not been waiting (just arrived)
                    qq.enqueue(pp, 0, num);
                    q.into(pp); // new process in queue
                    // System.out.println(title + " suspending process " +
                    //    pp.get_name());
                    pp.deactivate(pp);
                    return false;
                }
            }
        }
        System.out.println(title + " End of take " + pp.get_name());
        return true;
    }

    //
    /**
     * Returns number of available resource items in Bin object
     */
    public int num_avail() {
        return available;
    }

    //
    /**
     * Current process deposits a number of resource items in the Bin object
     * @param num number of items to deposit in Bin object
     */
    public void give(int num) {
        Process lpr; // process to dequeue
        Process pp = (Process) StaticSync.this_process();
        if (num <= 0) {
            Pobject.process_error(Pobject.E_BINNRR);
        }
        // System.out.println(title+ " Bin:give process " + pp.get_name() +
        //     " at: " + StaticSync.get_clock());
        available = available + num;
        if (available > max_bin) {
            max_bin = available;
        }
        //
        // Now get from waiting queue the first process with highest
        // priority
        //
        if (q.length() > 0) {
            lpr = (Process) q.out();
            if (lpr != null) {
                StaticSync.reactivate(lpr); // activate process
            }
        }
    }

    //
    private void use_bin() {
        double use_int = 0.0; // interval since last event
        double simclock = StaticSync.get_clock(); // time now
        if (obs_bin > 0) {
            use_int = simclock - lastev;
            free_int += use_int; // accum time
            freep += use_int * available; // product free time and available
        }
        lastev = simclock;
    }

    //
    // method to compute waiting statistics
    private void wait_bin(int req) {
        double req_int = 0.0; // interval since last event
        double simclock = StaticSync.get_clock(); // time now
        if (num_req > 0) { // resources that were requested
            // tot_proc += numproc;
            tot_req += num_req; // accum res requested
            req_int = simclock - rlastev;
            wait_time += req_int; // accumulate wait time
            prodr += req_int * num_req; // for average nr of req resources
            prodn += req_int * numproc; // for avg waiting processes
        }
        num_req += req;
        if (req > 0) {
            numproc++; // one more process requesting resources
        }
        else {
            numproc--;
        }
        if (numproc < 0) {
            numproc = 0;
        }
        if (num_req > 0) {
            rlastev = simclock; // current time
        }
    }

    //
    /**
     * Internal method for the actual reporting of Bin object usage, called by the
     * executive.
     */
    void report() {
        double simclock = StaticSync.get_clock();
        double yyd;
        // average number of instances of this resource held
        System.out.println(
            "-------------------------------------------------------");
        System.out.println(title);
        // System.out.println("Number of completed take operations: " + obs_bin);
        System.out.println("Max items: " + max_bin);
        if (free_int > 0) {
            yyd = ( (double) freep) / ( (double) free_int);
            System.out.println("Avg num available items: " + yyd);
        }
        else {
            System.out.println();
        }
        if (tot_req > 0) {
            yyd = ( (float) prodr) / ( (float) tot_req);
            System.out.println("Avg waiting time: " + yyd);
        }
        if (wait_time > 0) {
            yyd = ( (float) prodr) / ( (float) wait_time);
            System.out.println(", Avg num items requested: " + yyd);
            yyd = ( (float) prodn) / ( (float) wait_time);
            System.out.println("Avg. num waiting processes: " + yyd);
        }
        else {
            System.out.println();
        }
        // System.out.println( "------------------------------------------------------");
    }
} // end of Bin class
