package psimjava;

import java.util.*;
import java.net.*;
import java.lang.reflect.*;

/**
 * <p> Modeling Framework main Class</p>
 *
 * <p>Description: Abstract class that is the main control of the Modeling Framework.
 *  User must implement a subclass to control their Model</p>
 *
 * <p>Copyright: Copyright (c) 2004-2005</p>
 *
 * @author Rich Schlesinger
 * @version 1.0
 */
abstract public class Model extends psimjava.Process {
    private static String modelName;
    private static Model model;
    private static int animationDelay = 0;
    private Simulation sim;
    private UI ui;
    private double simPeriod;
    private static Class[] constructorParms = {};

    /**
     * Main Modeling Class
     * @param simIn Input - the Input Parameters
     */
    public Model(Input simIn) {
        super(modelName);
        model = this;
        ui = new UI(modelName, model, simIn);
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Main program for starting execution
     * @param args String[] - the command line arguments
     * @param title String - the name of this model
     */
    public static void main(String[] args, String title) {
        modelName = title;
        String className;
        try {
            Output.setOptions(args);
            try {
                throw new Exception();
            }
            catch (Exception e) {
                StackTraceElement[] callStack = e.getStackTrace();
                className = callStack[1].getClassName();
            }

            Class mainClass = Class.forName(className);
            Constructor c = mainClass.getConstructor(constructorParms);
            c.newInstance();
        }
        catch (Throwable e) {
            Output.reportError(e);
        }

    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Called after Input parameters set to setup Model
     * @param simIn Input parameters
     */
    public void modelBegin(Input simIn) {
        Output.report("Project: " + get_name());
        try {
            Output.report("Run at: " + new Date() + " by " +
                          System.getProperty("user.name") +
                          " on " + System.getProperty("os.name") + ", " +
                          InetAddress.getLocalHost().getCanonicalHostName());
        }
        catch (UnknownHostException e) {}

        // Record the Input Parameters
        Output.report("-----------------------------------------------------");
        Output.report("Input Parameters");
        String[] parms = simIn.getFormattedInputParameters();
        for (String p : parms) {
            Output.report(p);
        }
        sim = new Simulation(get_name());

        simPeriod = simIn.getSimPeriod();
        if (ui.isDisplayRequested()) {
            enableAnimation();
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Called to actually begin the simulation execution
     */
    public void startSimulation() {
        //
        try {
            Thread.sleep(100);
        } // initial sleep
        catch (InterruptedException e) {}
        //
        sim.start_sim(simPeriod);
        
        while (get_clock() < simPeriod) {
            try {
                Thread.sleep(500);
            } // initial sleep
            catch (InterruptedException e) {}
        }
        try {
            Thread.sleep(500);
        } // initial sleep
        catch (InterruptedException e) {}
        
        fileOut();

        ui.setSimulationDone();
    }

////////////////////////////////////////////////////////////////////////////
    /**
     * Called to Complete the execution
     */
    public void quit() {
        //       fileOut();
        System.exit(0);
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Produced report output
     */
    public void fileOut() {

        Output.report("-----------------------------------------------------");

        Output.report("Results of simulation: " + modelName);

        reportStatistics();
        Output.report("-----------------------------------------------------");

        Output.close();
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Method in subclass that is called after Input Parameters have been defined
     */
    abstract public void begin();

    /**
     * Method in subclass that is called to print any report statistics
     */
    abstract protected void reportStatistics();

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Return current value of simulation clock
     * @return double
     */
    public static double getCurrentClock() {
        return model.get_clock();
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * method called from within the Modeling Framework to determine delay for
     * animation
     * @return int
     */
    public static int getAnimationDelay() {
        return animationDelay;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * This method must be called if the Model is doing Animation
     */
    public static void enableAnimation() {
        animationDelay = 100;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Returns true if the Screen Display should be shown
     * @return boolean
     */
    public static boolean isDisplayRequested() {
        return (animationDelay!=0);
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * This method should be called periodically by a Process's Main_Body to
     * allow the screen display to be updated
     */
    public static void screenUpdate() {
        try {
            Thread.sleep(animationDelay);
        }
        catch (InterruptedException e) {

        }
    }
}
