package psimjava;

import java.text.*;
import java.io.*;
import java.awt.FileDialog;
import java.awt.Frame;

/**
 * <p> PSIM Modeling Framework Output class</p>
 *
 * <p>Description: Class to handle output to Trace and Summary Report files</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * @author Rich Schlesinger
 * @version 1.0
 */
public class Output {
    private static final DecimalFormat fmt = new DecimalFormat("0000.000");
    private static PrintStream tout = null;
    private static PrintStream out = null;
    private static boolean DEBUGGING;
    ////////////////////////////////////////////////////////////////////////////
    private Output(String simName, boolean newInstanceNeeded) {
        tout = getOutputFile("Trace", simName + " trace.txt", newInstanceNeeded);
        out = getOutputFile("Sum. Statistics", simName + " summary.txt", newInstanceNeeded);
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Method to create new instance of Print Files (Trace and Summary)
     * @param simName String - name of this simulation
     * @param newInstanceNeeded boolean
     */
    public static void newInstance(String simName, boolean newInstanceNeeded) {
        if (newInstanceNeeded || (tout == null)) {
            new Output(simName, newInstanceNeeded);
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Print a message in Trace file with the specified time
     * @param time double
     * @param msg String
     */
    public static void trace(double time, String msg) {
        String p = "Time " + fmt.format(time) + ' ' + msg;
        tout.println(p);
        if (DEBUGGING) {
            System.out.println(p);
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Print a message in Trace file
     * @param msg String
     */
    public static void trace(String msg) {
        tout.println(msg);
        if (DEBUGGING) {
            System.out.println(msg);
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * print a message in the Summary Report and Trace files
     * @param msg String
     */
    public static void report(String msg) {
        tout.println(msg);
        out.println(msg);
        if (DEBUGGING) {
            System.out.println(msg);
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Remembers the command line parameters
     * @param args String[]
     */
    public static void setOptions(String[] args) {
        for (String option : args) {
            if (option.equalsIgnoreCase("-d")) {
                DEBUGGING = true;
            }
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Reports an Exception during execution
     * @param e Throwable
     */
    public static void reportError(Throwable e) {
        if (tout != null) {
            e.printStackTrace(tout);
        }
        if (out != null) {
            e.printStackTrace(out);
        }
        if (DEBUGGING) {
            e.printStackTrace();
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    private PrintStream getOutputFile(String fileType,
                                      String fn,
                                      boolean newInstanceNeeded) {
        File file;
        String path = null;

        Frame fileWindow = new Frame();
        FileDialog fileChoice = new FileDialog
            (fileWindow, "Save " + fileType + " As", FileDialog.SAVE);
        fileChoice.setDirectory(System.getProperty("user.home")+ "\\My Documents");
        fileChoice.setFile(fn);
        fileChoice.setVisible(true);
        path = fileChoice.getDirectory();
        fn = fileChoice.getFile();

        file = (path == null) ? new File(fn) : new File(new File(path), fn);

        try {
            return new PrintStream(new FileOutputStream(file));
        }
        catch (FileNotFoundException e) {
            System.out.println("Cannot open file " + fn);
        }
        return null;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Close the Trace and Summary print files
     */
    public static void close() {
        out.close();
        tout.close();
    }

}
