package psimjava;

import java.awt.*;
import javax.swing.*;
import java.util.*;
/**
 * <p> Abstract class to create a Plot of a variable over time</p>
 *
 * <p>Copyright: Copyright (c) 2004-2005</p>
 *
 * @author M.E. Shaw, J. Garrido, R Schlesinger
 * @version 1.0
 */
abstract public class Plotter
    extends JFrame {
    private static final int FrameWidth = 800;
    private static final int FrameHeight = 400;

    private int scale; // Expected max vertical size
    private int jqlength; // current number of jobs in memory
    private double simTime; // elapsed run time
    private double simperiod; // total run time set at start
    private ArrayList<Point> pt = new ArrayList<Point> (); // stores references to trace points
    private Dimension dim; // for x, y coordinate axes
    private int xorigin, yorigin;

    private String yAxisLabel;
    private String[] legend;
    private int legendWidth;
    private int legendHeight;

    private int top;

    private JPanel qPlot = new JPanel();
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Create a Frame to hold the plot
     * @param title String
     * @param yAxisLabel String
     * @param legend String[]
     * @param x int - x coordinate for the plot
     * @param y int - y coordinate for the plot
     * @param simIn Input Parameters
     */
    public Plotter(String title,
                   String yAxisLabel,
                   String[] legend,
                   int x,
                   int y,
                   Input simIn) {
        super(title);

        this.legend = legend;
        this.yAxisLabel = yAxisLabel;

        xorigin = x / 10;
        yorigin = y * 8 / 10;

        simperiod = simIn.getSimPeriod();
        dim = new Dimension(x * 65 / 100, y * 55 / 100);

        setSize(FrameWidth, FrameHeight); //resize frame

        qPlot.setBackground(Color.white);
        setContentPane(qPlot);
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Defined in subclass to return the current value of the variable being plotted
     * @return int
     */
    abstract protected int getPlotY();

    /**
     * Defined in subclass to Return the maximum value for the variable being plotted
     * @return int
     */
    abstract protected int getScale();

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Called periodically by the Framework to update the plot
     * @param g Graphics
     */
    public void paint(Graphics g) {
        try {
            scale = getScale();
            if (legendHeight == 0) {
                getLegendSize(g);
                xorigin = xorigin + legendWidth;
                pt.add(new Point(xorigin, yorigin));
            }
            top = yorigin - dim.height * (scale + 1) / scale;
            g.setColor(Color.black);
            drawCoordinates(g);
            g.setColor(Color.blue);
            plotFunction(g);
            g.setColor(Color.black);
            drawLegend(g, 10, top);
        }
        catch (Throwable ex) {
            Output.reportError(ex);
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    private void plotFunction(Graphics g) {
        jqlength = getPlotY();
        // Adjust scale if necessary
        if (jqlength > scale) {
            scale = 2 * scale;
        }
        simTime = Model.getCurrentClock();

        int xx = xorigin + (int) (simTime * dim.width / simperiod);
        int yy = yorigin - (jqlength * dim.height) / scale;
        pt.add(new Point(xx, yy));

        for (int i = 1; i < pt.size(); i++) {
            g.drawLine(pt.get(i - 1).x, pt.get(i - 1).y,
                       pt.get(i).x, pt.get(i - 1).y);
            g.drawLine(pt.get(i).x, pt.get(i - 1).y,
                       pt.get(i).x, pt.get(i).y);
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    private void drawCoordinates(Graphics g) {
        //	Draw the x axis and tick marks
        int nx = (int) (simperiod / 10);
        g.drawLine(xorigin, yorigin, xorigin + dim.width * (nx + 10) / nx,
                   yorigin);
        String mssgX = "Elapsed time";
        g.drawString(mssgX, xorigin + dim.width, yorigin + 25);
        for (int i = 0; i <= nx + 10; i++) {
            if (i % 10 < 1) {
                mssgX = Integer.toString( (int) (i * simperiod / nx));
                g.drawString(mssgX, xorigin + dim.width * i / nx - 15,
                             yorigin + 15);
            }
            g.drawLine(xorigin + dim.width * i / nx, yorigin + 2,
                       xorigin + dim.width * i / nx, yorigin - 2);
        }

        //	Draw the y axis and tick marks
        int ny = scale;
        g.drawLine(xorigin, yorigin, xorigin, top);

        g.drawString(yAxisLabel, xorigin - 40, yorigin - dim.height - 40);  // 

        int increment = (scale >= 20) ? scale / 20 : 1;
        for (int i = 0; i <= ny + 1; i = i + increment) {
            if (i % (scale / 4) < 1) {
                String mssgY = Integer.toString(i);
                g.drawString(mssgY, xorigin - 20,
                             yorigin - dim.height * i / ny + 5);
            }
            g.drawLine(xorigin - 2, yorigin - dim.height * i / ny, xorigin + 2,
                       yorigin - dim.height * i / ny);
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    private void getLegendSize(Graphics g) {
        FontMetrics m = g.getFontMetrics();
        legendWidth = 0;

        for (String l : legend) {
            if (m.stringWidth(l) > legendWidth) {
                legendWidth = m.stringWidth(l);
            }
        }
        legendHeight = m.getMaxAscent() + m.getMaxDescent() + 1;
    }

    ////////////////////////////////////////////////////////////////////////////
    private void drawLegend(Graphics g, int x, int y) {
        for (String l : legend) {
            g.drawString(l, x, y);
            y = y + legendHeight;
        }
    }
}
