package psimjava;

/**
 * <p> PSIM Modeling Framework - Priority based Schedular/p>
 *
 * <p>Description: Schedular that provides the next highest priority Request</p>
 *
 * <p>Copyright (c) 2005</p>
 * @author Rich Schlesinger
 * @version 1.0
 */
public class PrioritySchedular extends Schedular {
    private Schedular[] schedulars;
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Create a PrioritySchedular with the specified number of priorities and queue size.
     * Each priority level is handled on a first-come first-server basis
     * @param name String
     * @param numberPriorities int
     * @param qsize int
     */
    public PrioritySchedular(String name, int numberPriorities, int qsize) {
        super(name);
        schedulars = new Schedular[numberPriorities];
        for (int i=0; i<numberPriorities; i++) {
            schedulars[i] = new FCFSschedular(name,qsize);
        }
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Create a PrioritySchedular using the specified Schedulars for the various priority levels
     * @param name String
     * @param schedulars Schedular[]
     */
    public PrioritySchedular(String name, Schedular... schedulars) {
        super(name);
        this.schedulars = schedulars;
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Schedule a Request at its specified priority level
     * @param newRequest T
     * @throws RejectedRequestException
     */
    public void schedule(Request newRequest) throws RejectedRequestException {
        int priority = newRequest.getRequestPriority();
        if (priority < 0) {
            throw new RejectedRequestException("Invalid Priority");
        }
        priority = Math.min(priority, schedulars.length);
        schedulars[priority].schedule(newRequest);
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the highest priority Request that is available
     * @return Request
     */
    public Request getNextRequest() {
        if (!isAccessAllowed()) return null;

        for (Schedular sched : schedulars) {
            if (sched.getNumberWaitingRequests() > 0) {
                Request rqst = sched.getNextRequest();
                updateSchedularInfo(rqst);
                return rqst;
            }
        }
        return null;
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Indicate processing of a request has completed
     * @param rqst Request - Completed Request
     */
    public void requestComplete(Request rqst) {
        int priority = rqst.getRequestPriority();
        priority = Math.min(priority, schedulars.length);
        requestComplete(rqst);
        schedulars[priority].requestComplete(rqst);

    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Return the total number of waiting requests
     * @return int
     */
    public int getNumberWaitingRequests() {
        int count = 0;
        for (Schedular sched : schedulars) {
            count += sched.getNumberWaitingRequests();
        }
        return count;
    }
}
