package psimjava;

import java.awt.*;

/**
 *
 * <p> Speciallized Process for handling Requests</p>
 *
 * <p>Description: A common type of Process is a RequestProcessor,
 *  which repeatedly takes in Requests and performs some service based on the
 *  request. Examples include a CPU and a Web Server.
 *  A RequestProcessor has a Schedular that determines the order in which the
 * Requests will be serviced. A Request is also a Process,
 *  RequestProcessor is an abstract class that the user must subclass to provide the desired
 * specific functionality (i.e. what the RequestProcessor actually does).</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * @author Rich Schlesinger
 * @version 1.0
 */
abstract public class RequestProcessor extends Process {
    // Simulation Parameters
    private double simPeriod;

    // References
    private Schedular sched;
    private Request currentRequest; // Request currently being serviced
    private Statistic serverBusy;
    private Statistic requestWaitTime;
    private RequestProcessor thisProcessor;

    // Current values
    private double busyStart = 0.0;
    private double idleStart; // time of start of idle
    private int active = 0;

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Create a RequestProcessor
     * @param name String name
     * @param sIn Input
     * @param sched Schedular to use with this Requestprocessor
     */
    public RequestProcessor(String name, Input sIn, Schedular sched) {
        this(name, sIn);
        this.sched = sched;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Create a RequestProcessor
     * @param name String
     * @param sIn Input
     */
    public RequestProcessor(String name, Input sIn) {
        super(name);
        simPeriod = sIn.getSimPeriod();
        serverBusy = new Statistic(name + " Busy Time");
        requestWaitTime = new Statistic(name + " Wait Time");
        thisProcessor = this;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Sepecifies the Schedular to be used by this RequestProcessor
     * @param sched Schedular
     */
    protected void setSchedular(Schedular sched) {
        this.sched = sched;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Called to schedule a Request for this RequestProcessor
     * @param request Request
     * @throws RejectedRequestException
     */
    public void schedule(Request request) throws RejectedRequestException {
        request.setWaitStartTime(get_clock());
        sched.schedule(request);
        if (thisProcessor.idle()) {
            reactivate(thisProcessor);
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the number of Requests currently waiting for this RequestProcessor
     * @return int
     */
    public int getNumberWaitingRequests() {
        return sched.getNumberWaitingRequests();
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Gets the next request for this RequestProcessor
     * @return Request
     */
    //protected Request getNextRequest() {
    // return (Request) sched.getNextRequest();
    //}

    ////////////////////////////////////////////////////////////////////////////

    /**
     * main execution loop for a RequestProcessor. Called by the underlying Process.
     */
    public void Main_body() {
        try {
            while (get_clock() <= simPeriod) {
                currentRequest = sched.getNextRequest();
                if (currentRequest == null) {

                    // No pending requests, so we are going IDLE
                    deactivate(this);
                }
                else {

                    active++;
                    if (active == 1) {
                        busyStart = get_clock();
                    }
                    requestWaitTime.accumulate(get_clock() -
                                               currentRequest.getWaitStartTime());
                    Output.trace(get_clock(),
                                 getName() + " servicing " +
                                 currentRequest.get_name());
                    serviceRequest(currentRequest);

                    delay(0.0);

                    // Delay for animation
                    Model.screenUpdate();
                }
            }
            // Record our last busy time
            // idleStart also represents the final time that was actually used
            idleStart = get_clock();
            serverBusy.accumulate(idleStart - busyStart);
            terminate();
        }
        catch (Throwable ex) {
            Output.reportError(ex);
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Method to indicate that a request has been processed
     * @param rqst Request Completed request
     */
    public synchronized void requestComplete(Request rqst) {
        active--;
        if (active == 0) {
            idleStart = get_clock();
            double busyTime = idleStart - busyStart;
            serverBusy.accumulate(busyTime);
            Output.trace(idleStart, get_name() + " goes idle ");
        }
        sched.requestComplete(rqst);
        if (thisProcessor.idle()) {
            reactivate(thisProcessor);
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Returns the number of Requests that are currently being Processed
     * @return int
     */
    public int getActiveCount() {
        return active;
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Concrete subclass must implement this method to service the specified Request
     * @param request Request
     */
    protected abstract void serviceRequest(Request request);

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Called to report statistics on the performance of this RequestProcessor
     */
    public void reportStatistics() {
        Output.report(get_name() + " Utilization: " +
                      serverBusy.formatRatio(idleStart, Statistic.PERCENT));
        Output.report(requestWaitTime.formatMean(Statistic.TIME));
        Output.report(requestWaitTime.formatMaximum(Statistic.TIME));
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Method that Animation can use to draw a RequestProcessor
     * @param g Graphics
     * @param x int
     * @param y int
     */
    public void drawGraphic(Graphics g, int x, int y) {
        final int qh = 25;
        g.setColor(active > 0 ? Color.RED : Color.LIGHT_GRAY);
        g.fillOval(x, y, qh, qh);
        g.setColor(Color.black);
    }

}
