package psimjava;

/**
 * PsimJ Java package for OO simulation
 * <P> Object-oriented simulation package, process interaction approach.
 * <P> (C) J. M. Garrido. June 2000, 2003, 2004
 *
 * Resource pools (containers).
 *
 * <P> The class 'Res' controls the declaration,
 * allocation and deallocation of resource pools. If a process requesting
 * a certain number of resource items from an object of type 'Res' and are
 * not available, then the process is enqueued into the resource's
 * priority queue. When a process releases a number of resource items,
 * the queue is examined for the waiting processes. A process
 * dequeued is checked if its request can be granted.
 *
 * <P> File: Res.java
 */
public class Res {
    private String title;
    private int max_res; // max number of resources of this type
    //
    private Pqueue q; // priority queue for waiting processes
    private Rqueue qq; // queue for resources held
    private int num_req; // current number of requested resources
    private int available; // nr of available resource objects
    //
    private double prod; // product nr of resources and usage time
    private double prodr; // product nr of res requested and wait time
    private double prodn; // product nr of processes requesting and wait time
    private double use_time; // accumulative time of resource usage
    private double wait_time; // accum wait time for resource
    private int tot_req; // accum resources requested
    //
    private double lastev; // time of last release/acquire for usage
    private double rlastev; // time of last request/acquire for waiting
    private int c_res; // number of resources used since last event
    private int numproc; // number of processes waiting
    //
    private int tot_proc; // accum number of processes waiting
    //
    /**
     * Constructor for the resurce pool object
     * @param name the title for the resource pool object
     * @param n the number of resource items in the resource pool
     */
    public Res(String name, int n) {
        title = new String(name); // copy string from name
        available = n;
        max_res = n;
        num_req = 0;
        prod = 0;
        prodr = 0;
        prodn = 0;
        use_time = 0;
        wait_time = 0;
        tot_req = 0;
        lastev = 0;
        rlastev = 0;
        c_res = 0;
        numproc = 0;
        tot_proc = 0;
        q = new Pqueue(name, Pobject.MAX_PRIORITIES); // initialize priority queue
        qq = new Rqueue(Pobject.MAX_PRIORITIES); // queue for resources held and requested
        Simulation.pt_res.addnode(this); // add pointer to this res object in res_chain
    }

    //
    /**
     * This method allows the requesting process to acquire a number of
     * resource items. If the resource pool does not have all these requested
     * resource items available, the requesting process is suspended.
     * @param num the number of resource items requested.
     */
    public void acquire(int num) {
        boolean res_cond;
        res_cond = false;
        while (!res_cond) {
            res_cond = bacquire(num);
        }
    }

    //
    private boolean bacquire(int num) { // process assumed 'running'
        Process lpr;
        Process pp;
        int resheld; // number of resources held
        boolean requested = false;
        pp = (Process) StaticSync.this_process(); // currently executing process
        if (num <= 0) {
            Pobject.process_error(Pobject.E_RESRI);
            // exit(1);
        }
        else { // still pend req res
            if (num > max_res) {
                Pobject.process_error(Pobject.E_RESRI);
                // exit(1);
            }
            resheld = qq.get_res_h(pp); // get num of res held
            // System.out.println(title+":Res:acquire, "+pp.get_name()+
            //          " res held: "+resheld);
            if (num > available) {
                // not sufficient resources for this process
                // put back process with its resource request num
                // at head of queue with priority
                if (qq.get_res_req(pp) > 0) { // already requesting before
                    // System.out.println(title+":acquire enqueue back: "+pp.get_name());
                    q.pback(pp);
                }
                else {
                    q.into(pp); // new process in queue
                    if (resheld == 0) {
                        qq.enqueue(pp, resheld, num);
                    }
                    else {
                        qq.up_res_req(pp, num);
                    }
                }
                // System.out.println(title+":acquire suspending " + pp.get_name()+
                //    " queue size: "+q.length());
                wait_res(num); // statistics on requested resources
                pp.deactivate(pp); // passivate process
                /*
                             System.out.println(title+" before wait");
                             try {
                             wait();
                             }
                             catch (InterruptedException e) {
                             e.printStackTrace();
                             }
                             System.out.println(title+" after wait");
                 */
                // System.out.println(title+" returning false");
                return false;
            }
            else {
                // ok, there are sufficient resources
                // if this process has been waiting to acquire resources
                // or if the waiting-processes queue is empty
                if (qq.get_res_req(pp) > 0 || q.length() == 0) {
                    // System.out.println(title+" "+pp.get_name()+" prev res req: "
                    //     +qq.get_res_req(pp)+ " queue length: "+q.length());
                    // allocate resource items
                    allocate_res(pp, num);
                    return true;
                }
                else {
                    // process has not been waiting (just arrived)
                    // scan queue of waiting processes to check if any waiting process
                    // requests res items that are currently available
                    lpr = scan_req(); // get process found
                    if (lpr == null) {
                        allocate_res(pp, num); // allocate res to current process
                        return true;
                    }
                    // System.out.println("Found: "+lpr.get_name()+" reactivating");
                    //
                    StaticSync.reactivate(lpr);
                    if (resheld == 0) {
                        qq.enqueue(pp, resheld, num);
                    }
                    else { // process has resources held
                        resheld = qq.up_res_req(pp, num); // update res held
                    }
                    q.into(pp); // new process in queue
                    // System.out.println(title+":acquire proc not been waiting; suspend "+
                    //        pp.get_name());
                    pp.deactivate(pp);
                    /*
                                    try {
                                    wait();
                                    }
                                    catch (InterruptedException e) {
                                    e.printStackTrace();
                                    }
                     */
                    return false;
                } // end else
            } // end else
        }
        return false;
    }

    //
    //
    // Scan queue of waiting processes for a process that requests res items
    // that are available
    private Process scan_req() {
        Process lpr;
        int lnreq;
        int jlen = q.length(); // number of processes waiting for res
        // System.out.println("Res:scan_req: scanning for processes with lower req"
        //    + " queue length: "+ jlen);
        while (jlen > 0) {
            lpr = (Process) q.out(); // dequeue a waiting process
            lnreq = qq.get_res_req(lpr); // res requested by lpr
            // System.out.println("Process: "+lpr.get_name()+" req: "+
            //     lnreq);
            if (lnreq <= available) {
                // process found with request that can be immediately be satisfied
                return lpr;
            }
            else {
                // return this process to the queue of waiting processes
                q.into(lpr);
            }
            jlen--;
        } // end while
        return null;
    } // end of scan_req

    //
    //
    //
    // allocate requested resource items to this process
    private void allocate_res(Process pp, int num) {
        Process lpr;
        int resheld;
        available = available - num; // update number of resources
        // add to number of resources held by this process
        resheld = qq.up_res_acq(pp, num);
        wait_res( -num); // for reporting requested resources
        use_res(); // for reporting resource usage
        // promote next waiting process
        // get next process in queue waiting for resources
        if (q.length() > 0) {
            lpr = (Process) q.out(); // process with highest priority
            // System.out.println(title+":Res:acquire, promoting "+
            //         lpr.get_name());
            StaticSync.reactivate(lpr); // promote to event list
        }
    }

    //
    /**
     * This method gets the number of available resource items available in the
     * resource pool.
     * @return int a number that represents the number of available
     * resource items.
     */
    public int num_avail() {
        return available;
    }

    //
    /**
     * This method gets the number of waiting processes. These processes have
     * been suspended because of their previuos resource requests.
     * @return int Number of Waiting processes
     */
    public int length() {
        return q.length();
    }

    //
    /**
     * This dunction releases resources from the calling process, and makes them
     * available in the resource pool. Then the method reactivates any waiting
     * processes.
     * @param num int, the number of resource items released by the calling process.
     */
    public void release(int num) {
        Process lpr; // process to dequeue
        Process pp;
        int res_left; // number of resource
        pp = (Process) StaticSync.this_process();
        if (num <= 0) {
            Pobject.process_error(Pobject.E_RESREL);
        }
        if ( (available + num) > max_res) {
            Pobject.process_error(Pobject.E_RESREL);
            // exit(1);
        }
        res_left = qq.get_res_h(pp); // resources held by process pp
        // System.out.println(title+" Res:release, res held "+res_left+
        //     " by " + pp.get_name());
        if (num > res_left) { // cannot release more than held
            System.out.println("Release more than held by " + pp.get_name());
            Pobject.process_error(Pobject.E_RESREL);
        }
        // update number of resources held by process on release
        res_left = qq.up_res_rel(pp, num);
        // System.out.println(title+":Res:release res held by "+ pp.get_name()+
        //           " after release: "+ res_left);
        available = available + num;
        use_res(); // for reporting resource usage
        //
        // Now get from waiting queue the first process with highest
        // priority
        // System.out.println(title+":Res:release, length of proc queue: " + q.length());
        // notifyAll();
        if (q.length() > 0) {
            lpr = (Process) q.out();
            // System.out.println( title+":Res:release reactivating " + lpr.get_name()+ " at "+
            //    StaticSync.get_clock());
            StaticSync.reactivate(lpr); // activate process
        }

        return;
    }

    //
    /**
     * Internal method.
     * Function for maintaining statistics for reporting usage and
     * average resources used.
     */
    void use_res() {
        //  static long lastev = 0;   // time of last relevant event
        //  static long use_time = 0; // accumulative time of resource usage
        //  static long prod = 0;   // product nr of resources and accum usage time
        double use_int = 0; // interval since last event
        //  static int c_res = 0;     // number of resources used since last event
        double simclock = StaticSync.get_clock();
        //
        if (c_res > 0) { // resources were used
            use_int = simclock - lastev;
            use_time += use_int; // accumulate usage time
            prod += use_int * c_res;
        }
        c_res = max_res - available; // current resources held
        if (c_res > 0) {
            lastev = simclock; // current time
        }
        return;
    }

    //
    /*
     * Internal method.
     * Function for maintaining statistics for reporting average wait period and
     * average processes waiting.
     */
    private void wait_res(int req) {
        // req is the number of requested resources now
        // if req is neg, the resource is assigned, then decrease the requested
        // number of resources.
        //  static long rlastev = 0;   // time of last relevant event
        //  static long wait_time = 0; // accumulative wait time for resource
        //  static long prodr = 0;   // product nr of resources requested and wait time
        double req_int = 0; // interval since last event
        //  int  num_req;  number of resources requested since last event
        //  int  tot_req; accum resources requested
        //  static int numproc = 0;   // number of processes waiting
        //  static int tot_proc = 0;   // accum nr of processes waiting
        double simclock = StaticSync.get_clock();
        //
        if (rlastev > 0) {
            tot_proc += numproc; // accum num of processes waiting
            tot_req += num_req; // accum res requested
            req_int = simclock - rlastev;
            if (num_req > 0) {
                wait_time += req_int; // accumulate wait time
            }
            prodr += req_int * num_req; // for average nr of req resources
            prodn += req_int * numproc; // for avg waiting processes
        }
        numproc = q.length(); // number of processes waiting
        num_req += req; // total num requested res for this period
        if (num_req < 0) {
            num_req = 0;
        }
        rlastev = simclock; // current time
    }

    //
    //
    /**
     * Internal method.
     * Function for the actual reporting, called by the executive
     */
    void report() {
        double simclock = StaticSync.get_clock();
        double timeres = simclock * max_res;
        // resource utilization
        System.out.println(
            "-----------------------------------------------------\n");
        System.out.println(title);
        if (timeres > 0) {
            System.out.println(title + " usage: " + (prod) / (timeres));
        }
        // average number of instances of this resource held
        if (use_time > 0) {
            System.out.println(" avg num items used: " + (prod) / (use_time) +
                               "\n");
            System.out.println("% of time spent resource usage: " +
                               (use_time) / (simclock));
        }
        else {
            System.out.println();
        }
        if (tot_proc > 0) {
            System.out.println("Avg waiting time: " + (prodn) / (tot_proc));
        }
        if (wait_time > 0) {
            // System.out.println(" Avg num items requested: " + (prodr)/(wait_time));
            System.out.println("Avg. num waiting processes: " +
                               (prodn) / (wait_time));
            System.out.println("% of time spent waiting: " +
                               (wait_time) / (simclock));
        }
        else {
            System.out.println();
        }
        // System.out.println("-----------------------------------------------------\n");
    }
} // end of class Res
