package psimjava;

import psimjava.*;
import java.awt.*;
import java.text.*;

/**
 * <p>A ResourceManager manages an inactive resource such as Memory.
 *  A Resource is an inactive entity.</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * @author Rich Schlesinger
 * @version 1.0
 */
public class ResourceManager
    extends Res {
    private String resourceName;
    private int resourceSize;
    private Statistic waitTime = new Statistic("Wait Time");
    private Statistic usage = new Statistic("Utilization");
    private Statistic jobsUsingResource = new Statistic("Jobs Using Resource");

    private final static NumberFormat percentFmt = NumberFormat.getPercentInstance();
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Creates a ResourceManager with the specified name and quantity of resources
     * @param name String
     * @param resourceSize int
     */
    public ResourceManager(String name, int resourceSize) {
        super(name, resourceSize);
        this.resourceName = name;
        this.resourceSize = resourceSize;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the total quantity of this resource
     * @return int
     */
    public int getResourceSize() {
        return resourceSize;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Acquire the specified amount of this resource. If the specified amount is
     * not available, the requesting Process is deactivated until the resource
     * becomes available.
     * @param amount int
     */
    public void acquire(int amount) {
        double waitStart = Model.getCurrentClock();
        super.acquire(amount);
        waitTime.accumulate(Model.getCurrentClock() - waitStart);
        usage.accumulate(resourceSize - num_avail());
        jobsUsingResource.accumulate(1);
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Release the specified quantity of this resource
     * @param amount int
     */
    public void release(int amount) {
        super.release(amount);
        usage.accumulate(resourceSize - num_avail());
        jobsUsingResource.accumulate(-1);
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the count of Processes currently using this resource
     * @return int
     */
    public int getProcessesUsingResource() {
        return (int)jobsUsingResource.getTotal();
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the percentage of this resource currently in use
     * @return double
     */
    public double getCurrentUsage() {
        return  (1 - num_avail() / (double)resourceSize);
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Reports usage statistics in the Summary file
     */
    public void reportStatistics() {
        Output.report(resourceName + ' ' + waitTime.formatMean(Statistic.TIME));
        Output.report(resourceName + ' ' + waitTime.formatMaximum(Statistic.TIME));
        Output.report(resourceName + " Mean Resource Usage " + percentFmt.format(usage.getMean() / resourceSize));
        Output.report(resourceName + " Maximum Resource Usage " + percentFmt.format(usage.getMaximum() / resourceSize));
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Draws an image of this Resource at the specified coordinates on the specified canvas
     * @param g Graphics
     * @param x int
     * @param y int
     */
    public void drawGraphic(Graphics g, int x, int y) {
        final int qh=25;
        g.setColor(new Color( (int) (255 * getCurrentUsage()), 0, 0));
        g.fillOval(x, y, qh, qh);
        g.setColor(Color.black);
    }

}
