package psimjava;

import java.util.*;

/**
  * <p> Generic Schedular class</p>
 *
 * <p>Copyright: Copyright (c) 2004-2005</p>
 *
 * @author R. Schlesinger
 * @version 1.0
 */
public abstract class Schedular {
    /**
     * Size of an Unbounded Queue
     */
    public static final int UNBOUNDED_SIZE = Integer.MAX_VALUE;
    /**
     * Currently pending Requests
     */
    protected LinkedList<Request> requests = new LinkedList<Request> ();
    /**
     * Maximum size of this queue
     */
    protected int qsize;
    /**
     * Maximum number of concurrent requests for the associated RequestProcessor
     */
    protected int maxConcurrentRequests = Schedular.UNBOUNDED_SIZE;
    /**
     * Current Request type (Exclusive or Shared)
     */
    protected int currentRequestType = 0;
    /**
     * Current number of active requests for the associated RequestProcessor
     */
    protected int currentRequestCount = 0;

    //////////////////////////////////////////////////////////////////////////////
    /**
     * Base class for Schedulars. All have a name
     * @param algorithm String - scheduling algorithm
     * @param qsize int - maximum size of the queue
     */
    public Schedular(String algorithm, int qsize) {
        this.qsize = qsize;
    }

    //////////////////////////////////////////////////////////////////////////////
    /**
     * Base class for Schedulars. Uses an Unbounded Queue
     * @param algorithm String - scheduling algorithm
     */
    public Schedular(String algorithm) {
        this.qsize = UNBOUNDED_SIZE;
    }

    //////////////////////////////////////////////////////////////////////////////
    /**
     * Called to schedule a new request.
     * Puts request on queue
     * @param newRequest Request
     * @throws RejectedRequestException if queue is full
     */
    public synchronized void schedule(Request newRequest) throws
        RejectedRequestException {
        if (requests.size() < qsize) {
            requests.add(newRequest);
        }
        else {
            throw new RejectedRequestException("Queue Full");
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the size of the queue used by this Schedular.
     * May return UNBOUNDED_QUEUE
     * @return int qsize
     */
    public int getQsize() {
        return qsize;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Returns number of pending requests
     * @return int
     */
    public int getNumberWaitingRequests() {
        return requests.size();
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Method (defined in specific subclass) to get the next request to be processed
     * @return Request
     */
    public abstract Request getNextRequest();

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Indicate processing of a request has completed
     * @param rqst Request Completed request
     */
    public synchronized void requestComplete(Request rqst) {
        //Note: currentRequestCount should always be > 0
        // This test is to prevent it from ever going negative.
        if (currentRequestCount > 0) {
            currentRequestCount--;
            if (currentRequestCount == 0) {
                currentRequestType = 0;
            }
        }

    }
    ////////////////////////////////////////////////////////////////////////////
    protected void updateSchedularInfo(Request rqst) {
        currentRequestCount++;
        currentRequestType = rqst.getRequestType();
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Returns true if a new request can be scheduled for the associated RequestProcessor
     * @return boolean
     */
    protected boolean isAccessAllowed() {
        return ((currentRequestType!=Request.EXCLUSIVE_ACCESS) &
               (currentRequestCount<maxConcurrentRequests) );
    }

}
