package psimjava;

/**
 * PsimJ Java package for Object Oriented simulation
 * <P>Object-oriented simulation package, process interaction approach.
 * <P> (C) J. M. Garrido. June 2000, 2003, 2004
 *
 * Simple queue (FIFO) class for PsimJ.
 * <P> Objects of this class are simple (FIFO) queues.

 */
public class Squeue {
    private String qname; // title for queue
    private int q_max; // Maximum number of objects in queue
    private int q_count; // current number of objects
    private Oqueue head; // head of queue
    private Oqueue tail; // tail of queue
    //
    /**
     * Constructor for initialicing objects of simple queues with a specified
     * name and capacity (number of objects).
     * @param a_name Title for the simple queue.
     * @param max_objects Maximum number of objects (capacity of the queue).
     */
    public Squeue(String a_name, int max_objects) { // constructor
        qname = a_name;
        q_max = max_objects;
        q_count = 0;
        head = null;
        tail = null;
    }

    // overloaded constructor
    /**
     * Constructor for simple queue for initialicing queues with specified name.
     * The default capacity is 1000 objects.
     * @param a_name Title for the simple queue.
     */
    public Squeue(String a_name) { // constructor
        qname = a_name;
        q_max = 2500; // default capacity of queue
        q_count = 0;
        head = null;
        tail = null;
    }

    //////////////////////////////////////////////////////////////////////////
    public String get_name() {
        return qname;
    }

    // nested inner class for linkable objects
    private class Oqueue {
        public Object myobj;
        public Oqueue next;
        public Oqueue prev;
        public Oqueue(Object v) {
            myobj = v;
            next = null;
            prev = null;
        }
    } // end of Oqueue

    /**
     * Get current number of objects in queue (queue size)
     * @returns Returns the current size of the queue.
     */
    public synchronized int length() {
        return q_count;
    }

    /**
     * Check if the queue is full.
     * @returns Returns 'true' if the queue is full, otherwise 'false'.
     */
    public synchronized boolean full() {
        if (q_max == q_count) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Check if queue is empty.
     * @returns Return 'true' if queue is empty, otherwise 'false'.
     */
    public synchronized boolean empty() {
        if (q_count == 0) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Insert specified object to the tail of the queue.
     * @param p Object (or process) to enqueue.
     */
    public synchronized void into(Object p) {
        // insert p at tail of queue
        if (q_max == q_count) { // is queue full ?
            Pobject.process_error(Pobject.E_PUTFULL);
        }
        Oqueue lobj = new Oqueue(p);
        if (q_count == 0) {
            head = lobj;
            tail = lobj;
        }
        else { // place new lobj after tail
            tail.next = lobj;
            lobj.prev = tail;
            tail = lobj;
        }
        q_count++;
    }

    /**
     * Insert the specified object to the head of the queue.
     * @param p Object (or process) to insert into the queue.
     */
    public synchronized void back(Object p) {
        // insert back at the head of the queue
        if (q_max == q_count) { // is queue full ?
            Pobject.process_error(Pobject.E_PUTFULL);
        }
        Oqueue lobj = new Oqueue(p);
        if (q_count == 0) {
            head = lobj;
            tail = lobj;
        }
        else {
            head.prev = lobj;
            lobj.next = head;
            head = lobj;
        }
        q_count++; // 4-19-01
    }

    /**
     * Dequeue an object or process from the head of queue.
     * @returns An object or process from the queue.
     */
    public synchronized Object out() {
        // dequeue an object from (at head) queue
        Object p;
        if (q_count == 0) { // is queue empty?
            Pobject.process_error(Pobject.E_GETEMPTY);
        }
        p = head.myobj;
        if (q_count == 1) {
            tail = null;
            head = null;
        }
        else {
            head = head.next; // new tail is prev lobj
            head.prev = null;
        }
        q_count--;
        return p;
    }

    /**
     * Remove the last object enqueued (the one at the tail of queue).
     * @returns Object (or process) from the queue.
     */
    public synchronized Object last() {
        // remove last object enqueued (from tail)
        Object p;
        if (q_count == 0) { // is queue empty?
            Pobject.process_error(Pobject.E_GETEMPTY);
        }
        p = tail.myobj;
        if (q_count == 1) {
            tail = null;
            head = null;
        }
        else {
            tail = tail.prev;
            tail.next = null;
        }
        q_count--;
        return p;
    }

    /**
     * Find and remove specified object or process from queue.
     */
    public synchronized void remov(Object pr) {
        // Find and remove one first occurrence of object pr from queue
        boolean found = false;
        boolean siga = true; // continue
        Oqueue qq = head;
        int mcount = 0; // number of objects examined
        if (q_count == 0) { // is queue empty?
            Pobject.process_error(Pobject.E_GETEMPTY);
        }
        while (siga == true && found == false) {
            mcount++;
            if (qq.myobj == pr) {
                found = true;
            }
            else {
                if (qq == tail) {
                    siga = false;
                }
                else {
                    qq = qq.next;
                }
            }
        }
        if (q_count == 1 && found == true) {
            tail = null;
            head = null;
            q_count = 0;
        }
        else if (q_count > 1 && found == true && mcount > 1) {
            if (qq.next == null) {
                qq.prev.next = qq.next; // unlink myobj
                tail = qq.prev;
            }
            else {
                qq.prev.next = qq.next; // unlink myobj
                qq.next.prev = qq.prev;
            }
            q_count--;
        }
        else if (q_count > 1 && mcount == 1 && found == true) {
            head = qq.next;
            head.prev = null;
            q_count--;
        }
    }
} // end of Squeue
