package psimjava;
import java.text.*;

/**
 * <p> This class maintains a single statistic.
 *  It accumulates the total and number of values.
 *  It can report Number Of Values, Total, Mean, and Ratio to some value
 *  Reported values can be returned as numbers or formatted strings</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * @author Rich Schlesinger
 * @version 1.0
 */
public class Statistic {
    /**
     * Format for displaying Time
     */
    public static final DecimalFormat TIME = new DecimalFormat("0000.000");
    /**
     * Format for displaying an integer
     */
    public static final DecimalFormat INTEGER = new DecimalFormat("#");
    /**
     * Format for displaying a percentage
     */
    public static final NumberFormat PERCENT = NumberFormat.getPercentInstance();

    private String name;
    private double total = 0.0;
    private double max = 0.0;
    private int numberValues = 0;

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Create a statistic
     * @param name String - statistic name for printing
     */
    public Statistic(String name) {
        this.name = name;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Accumulate an additional value for this Statistic
     * @param value double
     */
    public synchronized void accumulate(double value) {
        total += value;
        numberValues++;
        max = (value > max) ? value : max;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Return a count of the number of values accumulated
     * @return int
     */
    public int getNumberOfValues() {
        return numberValues;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the Total of the accumulated values
     * @return double
     */
    public double getTotal() {
        return total;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the maximum of the accumulated values
     * @return double
     */
    public double getMaximum() {
        return max;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the mean of the accumulated values
     * @return double
     */
    public double getMean() {
        return total / numberValues;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the ratio of the Total of the accumulated values and the specified denominator
     * @param denominator double
     * @return double
     */
    public double getRatio(double denominator) {
        return total / denominator;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Format the Total for printing, using the specified format
     * @param fmt NumberFormat
     * @return String
     */
    public String formatTotal(NumberFormat fmt) {
        return "Total " + name + ": " + fmt.format(total);
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Format the maximum value for printing, using the specified format
     * @param fmt NumberFormat
     * @return String
     */
    public String formatMaximum(NumberFormat fmt) {
        return "Maximum " + name + ": " + fmt.format(max);
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Format the mean for printing, using the specified format
     * @param fmt NumberFormat
     * @return String
     */
    public String formatMean(NumberFormat fmt) {
        return (numberValues==0) ? "N/A" : "Mean " + name + ": " + fmt.format(getMean());
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * format the Ratio, using the specified format
     * @param denominator double
     * @param fmt NumberFormat
     * @return String
     */
    public String formatRatio(double denominator, NumberFormat fmt) {
        return fmt.format(getRatio(denominator));
    }
}
