package psimjava;

// PsimJ simulation model in Java for Disk Scheduling
// This model includes graphic animation to partially illustrate
// in a visual manner the behavior of the batch system
// Model by R Schlesinger 2005
//
import javax.swing.*;
import java.awt.event.*; // import event interface
/**
 *
 * <p>Title: PSIM Modeling Framework</p>
 *
 * <p>Description: Class for creating Graphical User Interface for the
 * user to select Parameter values for the execution</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * @author Rich Schlesinger
 * @version 1.0
 */
class UI extends JFrame {
    private String title;
    private Input simInputs; // all input data from user
    private Model simulation;
    private Box buttonPanel = Box.createHorizontalBox();
    private JCheckBox doDisplay;
    private JButton startButton;
    private JButton quitButton;
    private JButton filesButton;
    private JLabel simulationDone;
    private boolean started = false;
    private static final int MAX_COLUMNS = 10;
    ////////////////////////////////////////////////////////////////////////////
    public UI(String title, Model model, Input simInputs) {
        super(title);
        this.title = title;
        this.simulation = model;
        this.simInputs = simInputs;

        doDisplay = addCheckBox("Do Display",
                                simInputs.DISPLAY_SUPPORTED,
                                simInputs.DISPLAY_SUPPORTED);
        startButton = addButton("Start", new StartButtonListener()); // start simulation
        quitButton = addButton("Quit", new QuitButtonListener()); // stop and quit simulation
        filesButton = addButton("Set Files", new FilesButtonListener()); // stop and quit simulation
        simulationDone = addLabel("Simulation Done",false);
        simInputs.getParameters();

        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
        mainPanel.add(Box.createVerticalStrut(20));
        mainPanel.add(buttonPanel);
        mainPanel.add(Box.createVerticalStrut(20));
        mainPanel.add(buildInputPanel(simInputs.parameters));
        mainPanel.add(Box.createVerticalStrut(20));
        getContentPane().add(mainPanel);
        pack();
        setVisible(true);
    }

    ////////////////////////////////////////////////////////////////////////////
    private JButton addButton(String name, ActionListener l) {
        JButton button = new JButton(name);
        button.addActionListener(l); // start simulation
        buttonPanel.add(button);
        buttonPanel.add(Box.createHorizontalStrut(20));
        return button;
    }
    ////////////////////////////////////////////////////////////////////////////
    private JCheckBox addCheckBox(String name, boolean checked, boolean visible) {
        JCheckBox cbox = new JCheckBox(name);
        cbox.setSelected(checked);
        cbox.setVisible(visible);
        buttonPanel.add(cbox);
        buttonPanel.add(Box.createHorizontalStrut(20));
        return cbox;
    }
    ////////////////////////////////////////////////////////////////////////////
    private JLabel addLabel(String text, boolean visible) {
        JLabel label = new JLabel(text);
        label.setVisible(visible);
         buttonPanel.add(label);
        buttonPanel.add(Box.createHorizontalStrut(20));
        return label;
    }
    ////////////////////////////////////////////////////////////////////////////
    private JComponent buildInputPanel(Parameter ...parms) {
        final int HSTRUT = 20;
        final int VSTRUT = 20;
        int numberRows = (parms.length + MAX_COLUMNS - 1) / MAX_COLUMNS;
        int numberColumns = (parms.length + numberRows - 1) / numberRows;
        Box vBox = Box.createVerticalBox();
        for (int row = 0; row < numberRows; row++) {
            vBox.add(vBox.createVerticalStrut(VSTRUT));
            Box hBox = Box.createHorizontalBox();
            hBox.add(Box.createHorizontalStrut(HSTRUT));
            int maxIndex = Math.min(parms.length, (row + 1) * numberColumns);
            for (int index = row * numberColumns; index < maxIndex; index++) {
                Parameter parm = parms[index];
                hBox.add(parm.getGUI());
                hBox.add(hBox.createHorizontalStrut(HSTRUT));
            }
            vBox.add(hBox);
        }
        JScrollPane s = new JScrollPane(vBox);
        return s;
    }

    ////////////////////////////////////////////////////////////////////////////
    //Overridden so we can exit when window is closed
    protected void processWindowEvent(WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
            System.exit(0);
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    // response to pressing "Start" button
    private class StartButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
          try {
            Output.newInstance(title, false);
            startButton.setEnabled(false);
            filesButton.setEnabled(false);
            simulation.begin();
            started = true;
          }
          catch (Throwable ex) {
            Output.reportError(ex);
          }
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    // When user clicks on the "Quit" button, exit program
    private class QuitButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (started) {
                simulation.quit();
            }
            else {
                System.exit(-1);
            }
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    // response to pressing "Set Files" button
    private class FilesButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Output.newInstance(title, true);
        }
    }
    ////////////////////////////////////////////////////////////////////////////
    public void setSimulationDone() {
        simulationDone.setVisible(true);
    }
    ////////////////////////////////////////////////////////////////////////////
    public boolean isDisplayRequested() {
        return doDisplay.isSelected();
    }
}
